------------------------------------------
-- Setting up libraries
------------------------------------------

local AddonName = "iWait"
iWait = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local QTip = LibStub("LibQTip-1.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local Tooltip;

iWait.Broker = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",

	OnClick = function(anchor, button)
		if( not iWait.Queued ) then
			
			if( button == "LeftButton" ) then
				ToggleLFDParentFrame();
			elseif( button == "MiddleButton" ) then
				ToggleFrame(RaidParentFrame); -- ToggleLFRParentFrame() doesn't seem to work
			elseif( button == "RightButton" ) then
				TogglePVPFrame();
			end
			
		else
		
			if( iWait.IsPvP ) then
				if( button == "LeftButton" and iWait.PvPState == 3 ) then
					ToggleWorldStateScoreFrame();
				elseif( button == "RightButton" ) then
					ToggleDropDownMenu(1, nil, MiniMapBattlefieldDropDown, anchor, 0, 0);
				end
			else
				if( button == "RightButton") then
					ToggleDropDownMenu(1, nil, MiniMapLFGFrameDropDown, anchor, 0, 0);
				end
			end
			
		end
	end,

	OnEnter = function(anchor)
		if( not iWait.Queued ) then
			return;
		end
		
		if( iWait.IsPvP ) then
			-- no PvP tooltip atm
		else
			LFGSearchStatus:SetParent(anchor);
			LFGSearchStatus:SetClampedToScreen(true);
			LFGSearchStatus:SetPoint("TOPRIGHT", anchor, "BOTTOM", 175, 0);
			LFGSearchStatus:Show();
		end
	end,
	
	OnLeave = function()
		if( iWait.IsPvP ) then
			-- no PvP tooltip atm
		else
			LFGSearchStatus:Hide();
		end
	end,
});

------------------------------------------
-- Variables, functions and colors
------------------------------------------

local COLOR_GOLD = "|cfffed100%s|r";
local COLOR_GRAY = "|cffaaaaaa%s|r";
local COLOR_RED  = "|cffff0000%s|r";
local COLOR_GREEN= "|cff00ff00%s|r";
local COLOR_YELLOW= "|cffffff00%s|r";

iWait.Queued = false;
iWait.LFGActive = false;
iWait.IsRaid = false; -- determines whether we queued for normal dungeon (lfd) or raid (lfr)
iWait.IsPvP = false;
iWait.IsArena = false;

iWait.PvPState = 0;
local PvPStates = {
	active = 3,
	confirm = 2,
	queued = 1,
	none = 0,
	error = 0,
};

local WorldPvPAlert = {};

local function LFGStatusColor(text, need, current)
	if( current < need ) then
		return (COLOR_GREEN):format(text);
	else
		return (COLOR_RED):format(text);
	end
end

local function LFGStatus(tank, heal, dps)
	if( iWait.IsRaid ) then
		return ("%s: %s%s%s"):format(
			L["LFR"],
			LFGStatusColor(L["T"], 1, tank),
			LFGStatusColor(L["H"], 1, heal),
			LFGStatusColor(L["D"], 1, dps)
		);
	else
		return ("%s: %s%s%s%s%s"):format(
			L["LFD"],
			LFGStatusColor(L["T"], 1, tank),
			LFGStatusColor(L["H"], 1, heal),
			LFGStatusColor(L["D"], 3, dps),
			LFGStatusColor(L["D"], 2, dps),
			LFGStatusColor(L["D"], 1, dps)
		);
	end
end

local function BrokerText(text)
	iWait.Broker.text = text;
end

local function BrokerIcon(icon)
	if( icon == "lfd" ) then
		iWait.Broker.icon = "Interface\\Addons\\iWait\\LFD-Green";
	elseif( icon == "lfd-disabled") then
		iWait.Broker.icon = "Interface\\Addons\\iWait\\LFD-Gray";
	elseif( icon == "pvp") then
		iWait.Broker.icon = "Interface\\BattlefieldFrame\\UI-Battlefield-Icon";
	end
end

------------------------------------------
-- OnInitialize and ResetBroker
------------------------------------------

function iWait:OnInitialize()
	self:ResetBroker();
	
	self:RegisterEvent("LFG_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_QUEUE_STATUS_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_PROPOSAL_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_PROPOSAL_SUCCEEDED", "LFGWait");
	
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "PvPWait");
end

function iWait:ResetBroker()
	BrokerText("");
	BrokerIcon("lfd-disabled");
	self.Queued = false;
	self.LFGActive = false;
	self.IsPvP = false;
	self.IsRaid = false;
	self.IsArena = false;
	self.PvPState = 0;
end

------------------------------------------
-- EventHandler
------------------------------------------

function iWait:PvPWait()
	local state = 0;	
	
	for i = 1, MAX_WORLD_PVP_QUEUES do
		local status, _, _, _, _, teamSize, _ = GetBattlefieldStatus(i);
		
		if( PvPStates[status] > state ) then
			state = PvPStates[status];
		end
		
		if( teamSize ~= 0 ) then
			self.IsArena = true;
		end
	end
	
	if( state == PvPStates["active"] ) then
		self.Queued = false;
		BrokerIcon("lfd");
		BrokerText((COLOR_GREEN):format(L["In Progress"]));
	elseif( state == PvPStates["confirm"] ) then
		self.Queued = false;
		BrokerIcon("lfd");
		
		if( self.IsArena ) then
			BrokerText((COLOR_YELLOW):format(L["Enter Arena"]));
		else
			BrokerText((COLOR_YELLOW):format(L["Enter Battlefield"]));
		end
	elseif( state == PvPStates["queued"] ) then
		self.Queued = true;
		self.IsPvP = true;
		BrokerIcon("pvp");
		
		if( self.IsArena ) then
			BrokerText((COLOR_YELLOW):format(L["Arena"]));
		else
			BrokerText((COLOR_YELLOW):format(L["Battlefield"]));
		end
	else
		self:ResetBroker();
	end
	
	self.PvPState = state;
end

function iWait:LFGWait(event)
	local mode, submode = GetLFGMode();
	
	if( not mode or mode == "suspended" ) then
		self:ResetBroker();
		return;
	elseif( mode == "lfgparty" ) then
		BrokerText((COLOR_GREEN):format(L["In Progress"]));
		BrokerIcon("lfd");
		self.Queued = false;
		self.LFGActive = true;
		return;
	elseif( mode == "proposal" ) then
		if( submode and submode == "unaccepted" ) then
			BrokerText((COLOR_GREEN):format(L["Enter Dungeon"]));
			BrokerIcon("lfd");
		else
			BrokerText((COLOR_YELLOW):format(L["Waiting..."]));
		end
	end
	
	local hasData, _, tankNeed, healNeed, dpsNeed, iniType, _, _, _, _, _, _, _ = GetLFGQueueStats();
	
	if( mode == "queued" ) then
		self.Queued = true;
		BrokerIcon("lfd");
		
		if( hasData ) then
			if( iniType and iniType == 2 ) then
				self.IsRaid = true;
			end
			
			BrokerText(LFGStatus(tankNeed, healNeed, dpsNeed));
		else
			BrokerText((COLOR_YELLOW):format(L["Preparing..."]));
		end
	end
end